@@ -14,7 +14,6 @@
 
 #include "config.h"
 #include "gldns/parseutil.h"
-#include <strings.h>
 #include <sys/time.h>
 #include <time.h>
 #include <ctype.h>
@@ -52,7 +51,7 @@ static const int mdays[] = {
 static int
 is_leap_year(int year)
 {
-	return GLDNS_MOD(year,   4) == 0 && (GLDNS_MOD(year, 100) != 0
+	return GLDNS_MOD(year,   4) == 0 && (GLDNS_MOD(year, 100) != 0 
 	    || GLDNS_MOD(year, 400) == 0);
 }
 
@@ -61,7 +60,7 @@ leap_days(int y1, int y2)
 {
 	--y1;
 	--y2;
-	return (GLDNS_DIV(y2,   4) - GLDNS_DIV(y1,   4)) -
+	return (GLDNS_DIV(y2,   4) - GLDNS_DIV(y1,   4)) - 
 	       (GLDNS_DIV(y2, 100) - GLDNS_DIV(y1, 100)) +
 	       (GLDNS_DIV(y2, 400) - GLDNS_DIV(y1, 400));
 }
@@ -121,7 +120,7 @@ static void
 gldns_mon_and_mday_from_year_and_yday(struct tm *result)
 {
 	int idays = result->tm_yday;
-	const int *mon_lengths = is_leap_year(result->tm_year) ?
+	const int *mon_lengths = is_leap_year(result->tm_year) ? 
 					leap_year_mdays : mdays;
 
 	result->tm_mon = 0;
@@ -166,20 +165,20 @@ gldns_gmtime64_r(int64_t clock, struct tm *result)
 #endif /* SIZEOF_TIME_T <= 4 */
 
 static int64_t
-gldns_serial_arithmitics_time(int32_t time, time_t now)
+gldns_serial_arithmetics_time(int32_t time, time_t now)
 {
 	int32_t offset = time - (int32_t) now;
 	return (int64_t) now + offset;
 }
 
 struct tm *
-gldns_serial_arithmitics_gmtime_r(int32_t time, time_t now, struct tm *result)
+gldns_serial_arithmetics_gmtime_r(int32_t time, time_t now, struct tm *result)
 {
 #if SIZEOF_TIME_T <= 4
-	int64_t secs_since_epoch = gldns_serial_arithmitics_time(time, now);
+	int64_t secs_since_epoch = gldns_serial_arithmetics_time(time, now);
 	return  gldns_gmtime64_r(secs_since_epoch, result);
 #else
-	time_t  secs_since_epoch = gldns_serial_arithmitics_time(time, now);
+	time_t  secs_since_epoch = gldns_serial_arithmetics_time(time, now);
 	return  gmtime_r(&secs_since_epoch, result);
 #endif
 }
@@ -289,9 +288,9 @@ gldns_parse_escape(uint8_t *ch_p, const char** str_p)
 {
 	uint16_t val;
 
-	if ((*str_p)[0] && isdigit((*str_p)[0])  &&
-	    (*str_p)[1] && isdigit((*str_p)[1])  &&
-	    (*str_p)[2] && isdigit((*str_p)[2]))  {
+	if ((*str_p)[0] && isdigit((unsigned char)(*str_p)[0]) &&
+	    (*str_p)[1] && isdigit((unsigned char)(*str_p)[1]) &&
+	    (*str_p)[2] && isdigit((unsigned char)(*str_p)[2])) {
 
 		val = (uint16_t)(((*str_p)[0] - '0') * 100 +
 				 ((*str_p)[1] - '0') *  10 +
@@ -304,7 +303,7 @@ gldns_parse_escape(uint8_t *ch_p, const char** str_p)
 		*str_p += 3;
 		return 1;
 
-	} else if ((*str_p)[0] && !isdigit((*str_p)[0])) {
+	} else if ((*str_p)[0] && !isdigit((unsigned char)(*str_p)[0])) {
 
 		*ch_p = (uint8_t)*(*str_p)++;
 		return 1;
@@ -348,14 +347,14 @@ gldns_b32_ntop_base(const uint8_t* src, size_t src_sz, char* dst, size_t dst_sz,
 	const char* b32 = extended_hex ?  "0123456789abcdefghijklmnopqrstuv"
 					: "abcdefghijklmnopqrstuvwxyz234567";
 
-	size_t c = 0; /* c is used to carry partial base32 character over
+	size_t c = 0; /* c is used to carry partial base32 character over 
 		       * byte boundaries for sizes with a remainder.
 		       * (i.e. src_sz % 5 != 0)
 		       */
 
 	ret_sz = add_padding ? gldns_b32_ntop_calculate_size(src_sz)
 			     : gldns_b32_ntop_calculate_size_no_padding(src_sz);
-
+	
 	/* Do we have enough space? */
 	if (dst_sz < ret_sz + 1)
 		return -1;
@@ -403,10 +402,12 @@ gldns_b32_ntop_base(const uint8_t* src, size_t src_sz, char* dst, size_t dst_sz,
 
 		/* ........ ........ ....4444 4....... ........ */
 			 c =  src[3]         >> 7 ;
+		/* fallthrough */
 	case 3: dst[4] = b32[(src[2] & 0x0f) << 1 | c];
 
 		/* ........ .......3 3333.... ........ ........ */
 			 c =  src[2]         >> 4 ;
+		/* fallthrough */
 	case 2:	dst[3] = b32[(src[1] & 0x01) << 4 | c];
 
 		/* ........ ..22222. ........ ........ ........ */
@@ -414,6 +415,7 @@ gldns_b32_ntop_base(const uint8_t* src, size_t src_sz, char* dst, size_t dst_sz,
 
 		/* .....111 11...... ........ ........ ........ */
 			 c =  src[1]         >> 6 ;
+		/* fallthrough */
 	case 1:	dst[1] = b32[(src[0] & 0x07) << 2 | c];
 
 		/* 00000... ........ ........ ........ ........ */
@@ -424,22 +426,25 @@ gldns_b32_ntop_base(const uint8_t* src, size_t src_sz, char* dst, size_t dst_sz,
 		switch (src_sz) {
 			case 1: dst[2] = '=';
 				dst[3] = '=';
+				/* fallthrough */
 			case 2: dst[4] = '=';
+				/* fallthrough */
 			case 3: dst[5] = '=';
 				dst[6] = '=';
+				/* fallthrough */
 			case 4: dst[7] = '=';
 		}
 	}
 	return (int)ret_sz;
 }
 
-int
+int 
 gldns_b32_ntop(const uint8_t* src, size_t src_sz, char* dst, size_t dst_sz)
 {
 	return gldns_b32_ntop_base(src, src_sz, dst, dst_sz, 0, 1);
 }
 
-int
+int 
 gldns_b32_ntop_extended_hex(const uint8_t* src, size_t src_sz,
 		char* dst, size_t dst_sz)
 {
@@ -468,7 +473,7 @@ gldns_b32_pton_base(const char* src, size_t src_sz, uint8_t* dst, size_t dst_sz,
 				ch = *src++;
 				--src_sz;
 
-			} while (isspace(ch) && src_sz > 0);
+			} while (isspace((unsigned char)ch) && src_sz > 0);
 
 			if (ch == '=' || ch == '\0')
 				break;
@@ -538,15 +543,18 @@ gldns_b32_pton_base(const char* src, size_t src_sz, uint8_t* dst, size_t dst_sz,
 			/* ........ ........ ........ .55555.. ........ */
 			/* ........ ........ ....4444 4....... ........ */
 			dst[3] = buf[4] << 7 | buf[5] << 2 | buf[6] >> 3;
+			/* fallthrough */
 
 		case 5: /* ........ ........ ....4444 4....... ........ */
 			/* ........ .......3 3333.... ........ ........ */
 			dst[2] = buf[3] << 4 | buf[4] >> 1;
+			/* fallthrough */
 
 		case 4: /* ........ .......3 3333.... ........ ........ */
 			/* ........ ..22222. ........ ........ ........ */
 			/* .....111 11...... ........ ........ ........ */
 			dst[1] = buf[1] << 6 | buf[2] << 1 | buf[3] >> 4;
+			/* fallthrough */
 
 		case 2: /* .....111 11...... ........ ........ ........ */
 			/* 00000... ........ ........ ........ ........ */
@@ -573,7 +581,7 @@ gldns_b32_pton_base(const char* src, size_t src_sz, uint8_t* dst, size_t dst_sz,
 					ch = *src++;
 					src_sz--;
 
-				} while (isspace(ch));
+				} while (isspace((unsigned char)ch));
 
 				if (ch != '=')
 					return -1;
@@ -590,7 +598,7 @@ gldns_b32_pton(const char* src, size_t src_sz, uint8_t* dst, size_t dst_sz)
 }
 
 int
-gldns_b32_pton_extended_hex(const char* src, size_t src_sz,
+gldns_b32_pton_extended_hex(const char* src, size_t src_sz, 
 		uint8_t* dst, size_t dst_sz)
 {
 	return gldns_b32_pton_base(src, src_sz, dst, dst_sz, 1, 1);
@@ -638,7 +646,7 @@ int gldns_b64_ntop(uint8_t const *src, size_t srclength,
 		target[o+1] = b64[ ((src[i]&0x03)<<4) | (src[i+1]>>4) ];
 		target[o+2] = b64[ ((src[i+1]&0x0f)<<2) ];
 		target[o+3] = pad64;
-		i += 2;
+		/* i += 2; */
 		o += 4;
 		break;
 	case 1:
@@ -647,7 +655,7 @@ int gldns_b64_ntop(uint8_t const *src, size_t srclength,
 		target[o+1] = b64[ ((src[i]&0x03)<<4) ];
 		target[o+2] = pad64;
 		target[o+3] = pad64;
-		i += 1;
+		/* i += 1; */
 		o += 4;
 		break;
 	case 0:
